@@ -38,7 +38,7 @@ static char copyright[] =
 #endif /* not lint */
 
 #ifndef lint
-static char sccsid[] = "@(#)mail.local.c	8.28 (Berkeley) 9/13/96";
+static char sccsid[] = "@(#)mail.local.c	8.34 (Berkeley) 11/24/96";
 #endif /* not lint */
 
 /*
@@ -79,11 +79,32 @@ static char sccsid[] = "@(#)mail.local.c	8.28 (Berkeley) 9/13/96";
 #endif
 
 #if (defined(sun) && defined(__svr4__)) || defined(__SVR4)
-# define USE_LOCKF
-# define USE_SETEUID
+# define USE_LOCKF	1
+# define USE_SETEUID	1
 # define _PATH_MAILDIR	"/var/mail"
 #endif
 
+#if defined(_AIX)
+# define USE_LOCKF	1
+# define USE_VSYSLOG	0
+#endif
+
+#if defined(ultrix)
+# define USE_VSYSLOG	0
+#endif
+
+#if defined(__osf__)
+# define USE_VSYSLOG	0
+#endif
+
+#if defined(NeXT)
+# include <libc.h>
+# define _PATH_MAILDIR	"/usr/spool/mail"
+# define __dead		/* empty */
+# define S_IRUSR	S_IREAD
+# define S_IWUSR	S_IWRITE
+#endif
+
 /*
  * If you don't have flock, you could try using lockf instead.
  */
@@ -93,6 +114,10 @@ static char sccsid[] = "@(#)mail.local.c	8.28 (Berkeley) 9/13/96";
 # define LOCK_EX	F_LOCK
 #endif
 
+#ifndef USE_VSYSLOG
+# define USE_VSYSLOG	1
+#endif
+
 #ifndef LOCK_EX
 # include <sys/file.h>
 #endif
@@ -614,14 +639,14 @@ vwarn(fmt, ap)
 	(void)vfprintf(stderr, fmt, ap);
 	(void)fprintf(stderr, "\n");
 
-#if !defined(ultrix) && !defined(__osf__)
+#if USE_VSYSLOG
 	/* Log the message to syslog. */
 	vsyslog(LOG_ERR, fmt, ap);
 #else
 	{
 		char fmtbuf[10240];
 
-		(void) sprintf(fmtbuf, fmt, ap);
+		(void) vsprintf(fmtbuf, fmt, ap);
 		syslog(LOG_ERR, "%s", fmtbuf);
 	}
 #endif
